@@ -23,7 +23,7 @@ from obs.model import Options, PutObjectHeader, AppendObjectHeader, AppendObject
 from obs.model import Redirect, RoutingRule, Tag, TagInfo, Transition, NoncurrentVersionTransition, Rule, Versions
 from obs.model import Object, WebsiteConfiguration, Logging, CompleteMultipartUploadRequest, DeleteObjectsRequest
 from obs.model import ListMultipartUploadsRequest, GetObjectRequest, UploadFileHeader, Payer
-from obs.model import ExtensionHeader, FetchStatus
+from obs.model import ExtensionHeader, FetchStatus, BucketAliasModel, ListBucketAliasModel
 from obs.workflow import WorkflowClient
 from obs.crypto_client import CryptoObsClient
 from obs.obs_cipher_suite import CTRCipherGenerator
@@ -91,5 +91,7 @@ __all__ = [
     'WorkflowClient',
     'CryptoObsClient',
     'CTRCipherGenerator',
-    'CtrRSACipherGenerator'
+    'CtrRSACipherGenerator',
+    'BucketAliasModel',
+    'ListBucketAliasModel'
 ]
